package com.logicbig.example;

import javax.persistence.CacheRetrieveMode;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashMap;
import java.util.Map;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            System.out.println("-- persisting entities --");
            persistEntities();
            System.out.println("-- loading entities --");
            loadEntities();
            System.out.println("-- loading entities with CacheRetrieveMode.BYPASS --");
            loadEntities2();
            System.out.println("-- loading entities with CacheRetrieveMode.BYPASS for employee only --");
            loadEntities3();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEntities() {
        Employee employee = new Employee();
        employee.setName("Dawn");
        employee.setDept("IT");
        Task task = new Task();
        task.setDescription("UI Coding");
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee);
        em.persist(task);
        em.getTransaction().commit();
        System.out.println("Employee persisted: " + employee);
        System.out.println("Task persisted: " + task);
        em.close();
    }

    private static void loadEntities() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Employee employee = em.find(Employee.class, 1L);
        System.out.println("Employee loaded: " + employee);
        Task task = em.find(Task.class, 2L);
        System.out.println("Task loaded: " + task);
        em.close();
    }

    private static void loadEntities2() {
        EntityManager em = entityManagerFactory.createEntityManager();
        em.setProperty("javax.persistence.cache.retrieveMode", CacheRetrieveMode.BYPASS);
        Employee employee = em.find(Employee.class, 1L);
        System.out.println("Employee loaded: " + employee);
        Task task = em.find(Task.class, 2L);
        System.out.println("Task loaded: " + task);
        em.close();
    }

    private static void loadEntities3() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Map<String, Object> properties = new HashMap<>();
        properties.put("javax.persistence.cache.retrieveMode", CacheRetrieveMode.BYPASS);
        Employee employee = em.find(Employee.class, 1L, properties);
        System.out.println("Employee loaded: " + employee);
        Task task = em.find(Task.class, 2L);
        System.out.println("Task loaded: " + task);
        em.close();
    }
}